<?php

namespace Drupal\sitemap\Plugin\Sitemap;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Template\Attribute;
use Drupal\Core\Url;
use Drupal\sitemap\SitemapBase;

/**
 * Provides a sitemap for an taxonomy vocabulary.
 *
 * @Sitemap(
 *   id = "vocabulary",
 *   title = @Translation("Vocabulary"),
 *   description = @Translation("Vocabulary description"),
 *   settings = {
 *     "title" = "",
 *     "show_description" = FALSE,
 *     "show_count" = FALSE,
 *     "term_depth" = 9,
 *     "term_count_threshold" = 0,
 *     "rss_depth" = 0,
 *   },
 *   deriver = "Drupal\sitemap\Plugin\Derivative\VocabularySitemapDeriver",
 *   enabled = FALSE,
 *   vocabulary = "",
 * )
 */
class Vocabulary extends SitemapBase {

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);

    // Provide the menu name as the default title.
    $vid = $this->getPluginDefinition()['vocabulary'];
    $vocab = \Drupal::entityTypeManager()->getStorage('taxonomy_vocabulary')->load($vid);
    $form['title']['#default_value'] = $this->settings['title'] ?: $vocab->label();

    // @TODO: Can we get default settings from the annotation?
    $form['show_description'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show vocabulary description'),
      '#default_value' => isset($this->settings['show_description']) ? $this->settings['show_description'] : FALSE,
      '#description' => $this->t('When enabled, this option will show the vocabulary description.'),
    ];

    $form['show_count'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show node counts next to taxonomy terms'),
      '#default_value' => isset($this->settings['show_count']) ? $this->settings['show_count'] : FALSE,
      '#description' => $this->t('When enabled, this option will show the number of nodes in each taxonomy term.'),
    ];

    $form['term_depth'] = [
      // @TODO: Number type not submitting?
      //'#type' = 'number',
      '#type' => 'textfield',
      '#title' => $this->t('Term depth'),
      '#default_value' => isset($this->settings['term_depth']) ? $this->settings['term_depth'] : 9,
      //'#min' => 0,
      //'#max' => 9,
      '#size' => 3,
      '#description' => $this->t('Specify how many levels of taxonomy terms should be included. For instance, enter <em>1</em> to only include top-level terms, or <em>0</em> to include no terms. The maximum depth is <em>9</em>.'),
    ];

    $form['term_count_threshold'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Term count threshold'),
      '#default_value' => isset($this->settings['term_count_threshold']) ? $this->settings['term_count_threshold'] : 0,
      '#size' => 3,
      '#description' => $this->t('Only show taxonomy terms whose node counts are greater than this threshold. Set to <em>0</em> to disable.'),
    ];

    $form['rss_depth'] = [
      '#type' => 'textfield',
      '#title' => $this->t('RSS depth'),
      '#default_value' => isset($this->settings['rss_depth']) ? $this->settings['rss_depth'] : 0,
      '#size' => 3,
      '#maxlength' => 10,
      '#description' => $this->t('Specify how many levels of taxonomy terms should have a link to the default RSS feed included. For instance, enter <em>1</em> to include an RSS feed for the top-level terms, or <em>0</em> to not include a feed. The maximum depth is <em>9</em>.'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function view() {
    $vid = $this->pluginDefinition['vocabulary'];
    /** @var \Drupal\taxonomy\Entity\Vocabulary $vocabulary */
    $vocabulary = \Drupal::entityTypeManager()->getStorage('taxonomy_vocabulary')->load($vid);
    $content = [];

    if (isset($this->settings['show_description']) && $this->settings['show_description']) {
      $content[] = ['#markup' => $vocabulary->getDescription()];
    }

    $last_depth = -1;

    $threshold = $this->settings['term_count_threshold'];
    $maxDepth = $this->settings['term_depth'];
    $rssDepth = $this->settings['rss_depth'];

    $terms = [];
    if ($maxDepth) {
      $terms = \Drupal::entityTypeManager()
        ->getStorage('taxonomy_term')
        ->loadTree($vid, 0, $maxDepth);
    }

    // @TODO: Handling for Forum vs Vocab
    if (\Drupal::service('module_handler')->moduleExists('forum') && $vid == \Drupal::config('forum.settings')->get('vocabulary')) {
      $title = Link::fromTextAndUrl($this->settings['title'], Url::fromRoute('forum.index'))->toString();
      $forum_link = TRUE;
    }
    else {
      $title = $this->settings['title'];
      $forum_link = FALSE;
    }

    // @TODO: Refactor as item_list, theme individual items.
    $output = '';
    foreach ($terms as $term) {
      $term->count = sitemap_taxonomy_term_count_nodes($term->tid);
      if ($term->count < $threshold) {
        continue;
      }

      // Adjust the depth of the <ul> based on the change
      // in $term->depth since the $last_depth.
      if ($term->depth > $last_depth) {
        for ($i = 0; $i < ($term->depth - $last_depth); $i++) {
          $output .= "\n<ul>";
        }
      }
      elseif ($term->depth == $last_depth) {
        $output .= '</li>';
      }
      elseif ($term->depth < $last_depth) {
        for ($i = 0; $i < ($last_depth - $term->depth); $i++) {
          $output .= "</li>\n</ul>\n</li>";
        }
      }
      // Display the $term.
      $output .= "\n<li>";
      $term_item = '';
      if ($forum_link) {
        $term_item .= Link::fromTextAndUrl($term->name, Url::fromRoute('forum.page', ['taxonomy_term' => $term->tid]))->toString();
      }
      else {
        // Always link to the term.
        $term_item .= Link::fromTextAndUrl($term->name, Url::fromRoute('entity.taxonomy_term.canonical', ['taxonomy_term' => $term->tid]))->toString();
      }
      if ($this->settings['show_count']) {
        $span_title = \Drupal::translation()->formatPlural($term->count, '1 item has this term', '@count items have this term');
        $term_item .= " <span title=\"" . $span_title . "\">(" . $term->count . ")</span>";
      }

      if ($rssDepth && $rssDepth > $term->depth) {
        $feed_icon = [
          '#theme' => 'sitemap_feed_icon',
          '#url' => Url::fromRoute('view.taxonomy_term.feed_1', ['arg_0' => $term->tid])->toString(),
          '#name' => $this->t('@term', ['@term' => $term->name]),
        ];
        $term_item .= \Drupal::service('renderer')->render($feed_icon);

      }

      // @TODO: Document
      // Add an alter hook for modules to manipulate the taxonomy term output.
      \Drupal::moduleHandler()->alter(['sitemap_taxonomy_term', 'sitemap_taxonomy_term_' . $term->tid], $term_item, $term);

      $output .= $term_item;

      // Reset $last_depth in preparation for the next $term.
      $last_depth = $term->depth;
    }

    // Bring the depth back to where it began, -1.
    if ($last_depth > -1) {
      for ($i = 0; $i < ($last_depth + 1); $i++) {
        $output .= "</li>\n</ul>\n";
      }
    }

    $content[] = [
      '#markup' => $output,
    ];

    return [
      '#theme' => 'sitemap_item',
      '#title' => $title,
      '#content' => $content,
      '#sitemap' => $this,
      // @TODO: Does a vocabulary cache tag exist?
    ];
  }

}
