<?php

namespace Drupal\sitemap\Tests;

/**
 * Tests the display of taxonomies based on sitemap settings.
 *
 * @group sitemap
 */
class SitemapTaxonomyTermsTest extends SitemapTaxonomyTestBase {

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    // Create terms.
    $this->terms = $this->createTerms($this->vocabulary);
  }

  /**
   * Tests the term_threshold setting.
   */
  public function testTermThreshold() {
    // The vocabulary is already configured to display in parent ::setUp().
    $vocab = $this->vocabulary;
    $vid = $vocab->id();

    // Get term names from terms.
    $names = [];
    foreach ($this->terms as $term) {
      $names[] = $term->label();
    }

    // Confirm that terms without content are displayed by default.
    $this->drupalGet('/sitemap');
    foreach ($names as $term_name) {
      $this->assertLink($term_name);
    }

    // Create test node with terms.
    $this->createNodeWithTerms($this->terms);
    // @TODO: Figure out proper cache tags.
    drupal_flush_all_caches();

    // Require at least one node for taxonomy terms to show up.
    $this->saveSitemapForm(["plugins[vocabulary:$vid][settings][term_count_threshold]" => 1]);

    // Assert that terms with content are displayed on the sitemap as links.
    $this->drupalGet('sitemap');
    foreach ($names as $term_name) {
      $this->assertLink($term_name);
    }

    // Require at least two nodes for taxonomy terms to show up.
    $this->saveSitemapForm(["plugins[vocabulary:$vid][settings][term_count_threshold]" => 2]);

    $terms = $this->terms;
    unset($terms[0]);

    // Create a second test node with only two terms.
    $this->createNodeWithTerms($terms);

    $this->drupalGet('sitemap');
    $this->assertNoLink($this->terms[0]->label());
    $this->assertLink($this->terms[1]->label());
    $this->assertLink($this->terms[2]->label());
  }

  /**
   * Tests appearance of node counts.
   */
  public function testNodeCounts() {
    // The vocabulary is already configured to display in parent ::setUp().
    $vocab = $this->vocabulary;
    $vid = $vocab->id();

    // Create test node with terms.
    $this->createNodeWithTerms($this->terms);

    // Assert that node counts are not included in the sitemap by default.
    $this->drupalGet('/sitemap');
    $this->assertEqual(substr_count($this->getTextContent(), '(1)'), 0, 'Node counts are included');

    // Configure module to display node counts.
    $this->saveSitemapForm(["plugins[vocabulary:$vid][settings][show_count]" => TRUE]);

    // Assert that node counts are included in the sitemap.
    $this->drupalGet('sitemap');
    $this->assertEqual(substr_count($this->getTextContent(), '(1)'), 3, 'Node counts are not included');
  }

  /**
   * Tests vocabulary depth settings.
   */
  public function testVocabularyDepth() {
    // The vocabulary is already configured to display in parent ::setUp().
    $vocab = $this->vocabulary;
    $vid = $vocab->id();

    // Set to show all taxonomy terms, even if they are not assigned to any
    // nodes.
    $this->saveSitemapForm(["plugins[vocabulary:$vid][settings][term_count_threshold]" => -1]);

    // Get tags from terms.
    $tags = [];
    foreach ($this->terms as $term) {
      $tags[] = $term->label();
    }

    // Change vocabulary depth to its maximum (9).
    $this->saveSitemapForm(["plugins[vocabulary:$vid][settings][term_depth]" => 9]);

    // Assert that all tags are listed in the sitemap.
    $this->drupalGet('/sitemap');
    foreach ($tags as $tag) {
      $this->assertText($tag);
    }

    // Change vocabulary depth to 0.
    $this->saveSitemapForm(["plugins[vocabulary:$vid][settings][term_depth]" => 0]);

    // Assert that no tags are listed in the sitemap.
    $this->drupalGet('/sitemap');
    foreach ($tags as $tag) {
      $this->assertNoText($tag);
    }

    // Change vocabulary depth to 1.
    $this->saveSitemapForm(["plugins[vocabulary:$vid][settings][term_depth]" => 1]);

    // Assert that only tag 1 is listed in the sitemap.
    $this->drupalGet('/sitemap');
    $this->assertText($tags[0]);
    $this->assertNoText($tags[1]);
    $this->assertNoText($tags[2]);

    // Change vocabulary depth to 2.
    $this->saveSitemapForm(["plugins[vocabulary:$vid][settings][term_depth]" => 2]);

    // Assert that tag 1 and tag 2 are listed in the sitemap.
    $this->drupalGet('/sitemap');
    $this->assertText($tags[0]);
    $this->assertText($tags[1]);
    $this->assertNoText($tags[2]);

    // Change vocabulary depth to 3.
    $this->saveSitemapForm(["plugins[vocabulary:$vid][settings][term_depth]" => 3]);

    // Assert that all tags are listed in the sitemap.
    $this->drupalGet('/sitemap');
    foreach ($tags as $tag) {
      $this->assertText($tag);
    }
  }

}
